;atmega328 AVR programmer
;written 2011
;modified 17/10/2011
;modified 20/10/2011 - branch and jump problems sorted out
;modified 21/10/2011 - parallel programming routines
;
;
.include "m328pdef.inc"
;
.set usec10 = $100
.set msec = $101
.set ms10 = $102
.set secs = $103
.set tx_data = $104
.set rx_data = $105
.set bit_counter = $106
.set data_byte1 = $107
.set data_byte2 = $108
.set data_byte3 = $109
.set data_byte4 = $10a
.set byte_count = $10b

.def delay = r17
.def delay_value = r16
.def temp = r18
.def data_byte = r19
.def echo = r20

.set baud = 51	;9600baud clock 8mhz

.set chip_erase = $80
.set Write_fuse_bits = $40
.set write_lock_bits = $20
.set write_flash = $10
.set write_eeprom = $11
.set read_sig_cal_bytes = $08
.set read_fuse_lock_bits = $04
.set read_flash = $02
.set read_eeprom = $03

.set miso = 0		;PB0
.set mosi = 1		;PB1
.set sck = 2		;PB2
.set reset = 3		;PB3

.set rxd = 0		;PD0
.set txd = 1		;PD1
.set cts = 2		;PD2
.set rts = 3		;PD3

.set rdy = 6		;PC6

.set oe = 4			;PD4
.set wr = 5			;PC5
.set bs1 = 4		;PC4
.set bs = 4			;PC4
.set xa0 = 3		;PC3
.set xa1 = 2		;PC2
.set pagel = 1		;PC1
.set bs2 = 0		;PC0
.set xtal1 = 7		;PD7
.set vdd_en = 6		;PD6 - target avr VDD 
.set vpp_en = 5		;PD5 - target avr VPP 

.set data_out = portB
.set data_in = pinB


setup:

	ldi 	R16,low(RAMEND)		; Load low byte address of end of RAM into register R16
	out		SPL,temp			; Initialize stack pointer to end of internal RAM


	ldi 	R16,high(RAMEND)	; Load high byte address of end of RAM into register R16	
	out 	SPH,temp		; Initialize high byte of stack pointer to end of internal RAM

	ldi temp,0b10000011		;set up for 4mhz clock i.e 8mhz div by 2
	sts clkpr,temp

	ldi temp,0
	out ddrb,temp
	ldi temp,$3f
	out ddrc,temp
	ldi temp,$f6
	out ddrd,temp

	rcall wdt_off
	rcall usart_init

	cbi portd,oe
	cbi portd,xtal1
	cbi portc,wr
	cbi portc,bs1
	rcall set_din
;////////////////////

main:
;	ldi temp,0
;	out portb,temp
;	ldi delay,100
;	rcall delayms10
;
;	ldi temp,255
;	out portb,temp
;	ldi delay,100
;	rcall delayms10
;	rjmp main

	cbi portd,cts
	rcall serin
	sbi portd,cts
	lds temp,rx_data
	cpi temp,1
	brne pc+2
	rjmp set_prog_mode
	cpi temp,2
	brne pc+2
	rjmp prog_enable
	cpi temp,3
	brne pc+2
	rjmp set_reset
	cpi temp,4
	brne pc+2
	rjmp write_mode
	cpi temp,8
	brne pc+2
	rjmp read_mode
	cpi temp,9
	brne pc+2
	rjmp pulse_sck
	cpi temp,16
	brne pc+2
	rjmp enter_prog_mode
	cpi temp,17
	brne pc+2
	rjmp cmd_chip_erase
	cpi temp,18
	brne pc+2
	rjmp load_command
	cpi temp,19
	brne pc+2
	rjmp load_address_low
	cpi temp,20
	brne pc+2
	rjmp load_address_high
	cpi temp,21
	brne pc+2
	rjmp load_data_low
	cpi temp,22
	brne pc+2
	rjmp load_data_high
	cpi temp,23
	brne pc+2
	rjmp prog_page
	cpi temp,24
	brne pc+2
	rjmp prog_end
	cpi temp,25
	brne pc+2
	rjmp cmd_read_flash_low
	cpi temp,26
	brne pc+2
	rjmp cmd_read_flash_high
	cpi temp,27
	brne pc+2
	rjmp latch_data
	cpi temp,28
	brne pc+2
	rjmp prog_fuse_low
	cpi temp,29
	brne pc+2
	rjmp prog_fuse_high
	cpi temp,30
	brne pc+2
	rjmp vdd_on
	cpi temp,31
	brne pc+2
	rjmp vpp_on
	cpi temp,32
	brne pc+2
	rjmp vdd_off
	cpi temp,33
	brne pc+2
	rjmp vpp_off
	cpi temp,34
	brne pc+2
	rjmp prog_extend_fuse
	cpi temp,35
	brne pc+2
	rjmp prog_lock_byte
	cpi temp,36
	brne pc+2
	rjmp read_fuse_low_byte
	cpi temp,37
	brne pc+2
	rjmp read_fuse_high_byte
	cpi temp,38
	brne pc+2
	rjmp read_extended_fuse_byte
	cpi temp,39
	brne pc+2
	rjmp read_lock_byte
	cpi temp,254
	brne pc+2
	rjmp test_a
	rjmp main

read_mode:
	ldi temp,3
	rcall in_n_bytes
	ldi temp,3
	rcall write_n_bytes
	rcall read_data_byte
	lds temp,data_byte4
	sts tx_data,temp
	rcall serout
	rjmp main

write_mode:
	ldi temp,4
	rcall in_n_bytes
	ldi temp,4
	rcall write_n_bytes
	rjmp main


set_prog_mode:
	cbi portb,sck
	nop
	nop
	sbi portb,reset
	ldi delay,30
	rcall delayms10
	cbi portb,reset
	rjmp main

pulse_sck:
	rcall delay10
	sbi portb,sck
	rcall delay10
	cbi portb,sck
	rjmp main

pulse_sck1:
	rcall delay10
	sbi portb,sck
	rcall delay10
	cbi portb,sck
	ret

set_reset:	
	sbi portb,reset
	rjmp main

prog_enable:
	ldi data_byte,$ac
	rcall write_1_byte		;send byte 1
	ldi data_byte,$53
	rcall write_1_byte		;send byte 2
	ldi data_byte,0
	rcall write_1_byte		;send byte 3
	
	sts tx_data,echo		;save echo byte
	ldi data_byte,0
	rcall write_1_byte		;send byte 4
	rcall serout			;send byte 2 echo to PC.
	rjmp main

in_n_bytes:		;come in with byte count in temp
	sts byte_count,temp
	ldi temp,high(data_byte1)
	mov xh,temp
	ldi temp,low(data_byte1)
	mov xl,temp
in_loop:
	cbi portd,cts
	rcall serin
	sbi portd,cts
	lds temp,rx_data
	st x+,temp
	lds temp,byte_count
	dec temp
	sts byte_count,temp
	brne in_loop
	ret 


read_data_byte:
	ldi temp,8
	sts bit_counter,temp
loop_read:
	rcall delay10
	sbi portb,sck
	rcall delay10
	in temp,pinb
	clc
	sbrc temp,miso
	sec
	lds temp,data_byte4
	rol temp
	sts data_byte4,temp
	cbi portb,sck
	lds temp,bit_counter
	dec temp
	sts bit_counter,temp
	brne loop_read
	ret



write_1_byte:
	ldi temp,8
	sts bit_counter,temp	;set a bit count of 8
	clr echo				
write_loop1:
	sbrc data_byte,7		;test msb of data
	rjmp setbit
	cbi portb,mosi			;output a 0
	rjmp clock_bit
setbit:
	sbi portb,mosi			;output a 1

clock_bit:
	rcall delay10
	sbi portb,sck			;clock high
	rcall delay10
	 
	in temp,pinb
	clc
	sbrc temp,miso
	sec
	rol echo
	cbi portb,sck
	lsl data_byte
	lds temp,bit_counter
	dec temp
	sts bit_counter,temp
	brne write_loop1
	ret

write_n_bytes:			;byte_count in temp
	sts byte_count,temp
	ldi temp,high(data_byte1)
	mov xh,temp
	ldi temp,low(data_byte1)
	mov xl,temp
write_loop:
	ldi temp,8
	sts bit_counter,temp
loop_8_bits:
	ld temp,x
	sbrs temp,7
	rjmp n1	;pc + 3
	sbi portb,mosi
	rjmp n2	;pc + 2
n1:
	cbi portb,mosi
n2:
	nop
	rcall pulse_sck1
	nop
	lsl temp
	st x,temp
	lds temp,bit_counter
	dec temp
	sts bit_counter,temp
	brne loop_8_bits
	ld temp,x+			;increment x register
	lds temp,byte_count
	dec temp
	sts byte_count,temp
	brne write_loop
	ret

				

usart_init:
	ldi temp,0
	sts ubrr0h,temp
	ldi temp,baud
	sts ubrr0l,temp
	ldi temp,(1<<rxen0) | (1<<txen0)
	sts ucsr0b,temp
	ldi temp, (1<<usbs0) | (3<<ucsz00)
	sts ucsr0c,temp
	ret	

ser_flush:
	lds temp,ucsr0a
	sbrs temp,rxc0
	ret
	lds temp,udr0
	rjmp ser_flush

serout:
	lds temp,ucsr0a
	sbrs temp,udre0
	rjmp serout
	lds temp,tx_data
	sts udr0,temp
	ret

serin:
	lds temp,ucsr0a
	sbrs temp,rxc0
	rjmp serin
	lds temp,udr0
	sts rx_data,temp
	ret

get_ser_byte:
	cbi portd,cts
	rcall serin
	sbi portd,cts
	ret


;====================================

enter_prog_mode:
	cbi portd,vpp_en
	cbi portd,vdd_en
	cbi portc,pagel
	cbi portc,xa1
	cbi portc,xa0
	cbi portc,bs1
	rcall set_din
	sbi portd,oe
	cbi portd,xtal1
	cbi portc,wr
	cbi portc,bs2
	nop
	nop
	sbi portd,vdd_en
	ldi delay,5
	rcall delay10us
	sbi portd,vpp_en
	ldi delay,50
	rcall delay10us
	sbi portc,wr
	rjmp main

cmd_chip_erase:
	rcall set_dout
	sbi portc,wr
	sbi portc,xa1
	cbi portc,xa0
	cbi portc,bs1
	ldi temp,$80			;chip erase command
	out data_out,temp
;	nop
	nop
	sbi portd,xtal1
;	nop
	nop
	cbi portd,xtal1
;	nop
	nop
	cbi portc,wr
;	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	rcall set_din
	rjmp main

load_command:
	rcall set_dout
	rcall get_ser_byte
	sbi portc,xa1
	cbi portc,xa0
	cbi portc,bs1
	lds temp,rx_data
	out data_out,temp		;pre loaded  register with command
	nop
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main

load_address_low:
	rcall set_dout
	rcall get_ser_byte
	cbi portc,xa1
	cbi portc,xa0
	cbi portc,bs1
	lds temp,rx_data
	out data_out,temp
	nop			;pre load register with address low byte
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main

load_data_low:
	rcall set_dout				;data out as outputs
	rcall get_ser_byte
	cbi portc,xa1
	sbi portc,xa0
	lds temp,rx_data	
	out data_out,temp			;pre load register with data low byte
	nop
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main

load_data_high:
	rcall set_dout
	rcall get_ser_byte
	sbi portc,bs1
	cbi portc,xa1
	sbi portc,xa0
	lds temp,rx_data
	out data_out,temp			;pre load register with data high byte
	nop
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main

latch_data:
	sbi portc,bs1
	nop
	nop
	sbi portc,pagel
	nop
	nop
	cbi portc,pagel
	rjmp main

load_address_high:
	rcall set_dout
	rcall get_ser_byte
	cbi portc,xa1
	cbi portc,xa0
	sbi portc,bs1
	lds temp,rx_data
	out data_out,temp
	nop
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main
		
prog_page:
;	cbi portc,bs1
;	nop
	cbi portc,wr
	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	rjmp main

prog_end:
	sbi portc,xa1
	cbi portc,xa0
	rcall set_dout
	clr temp
	out data_out,temp
	nop
	nop
	sbi portd,xtal1
	nop
	nop
	cbi portd,xtal1
	rjmp main

cmd_read_flash_low:
	rcall set_din
	cbi portd,oe
	cbi portc,bs1
	nop
	nop
	nop
	nop
	nop
	in temp,data_in
	sts tx_data,temp
	rcall serout
	rjmp main

cmd_read_flash_high:
	rcall set_din
	cbi portd,oe
	sbi portc,bs1
	nop
	nop
	nop
	nop
	nop
	in temp,data_in
	sts tx_data,temp
	rcall serout
	sbi portd,oe
	rcall set_dout
	rjmp main

prog_fuse_low:
	cbi portc,bs1
	nop
	cbi portc,wr
	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	rjmp main

prog_fuse_high:
	sbi portc,bs1
	cbi portc,bs2
	nop
	cbi portc,wr
	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	cbi portc,bs1
	rjmp main

prog_extend_fuse:
	cbi portc,bs1
	sbi portc,bs2
	nop
	nop
	cbi portc,wr
	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	cbi portc,bs2
	rjmp main

prog_lock_byte:
	nop
	cbi portc,wr
	nop
	nop
	sbi portc,wr
	nop
	in temp,pinc
	sbrs temp,rdy
	rjmp pc-2
	rjmp main

read_fuse_low_byte:
	cbi portd,oe
	cbi portc,bs2
	cbi portc,xa1	;for at tiny
	cbi portc,bs1
	nop
	nop
	rcall set_din
	in temp,data_in
	sts tx_data,temp
	rcall serout
	sbi portd,oe
	rjmp main

read_fuse_high_byte:
	cbi portd,oe
	sbi portc,bs2
	sbi portc,xa1	;for at tiny
	sbi portc,bs1
	nop
	nop
	rcall set_din
	in temp,data_in
	sts tx_data,temp
	rcall serout
	sbi portd,oe
	rjmp main

read_extended_fuse_byte:
	cbi portd,oe
	sbi portc,bs2
	sbi portc,xa1	;for at tiny
	cbi portc,bs1
	nop
	rcall set_din
	nop
	in temp,data_in
	sts tx_data,temp
	rcall serout
	sbi portd,oe
	rjmp main

read_lock_byte:
	cbi portd,oe
	cbi portc,bs2
	cbi portc,xa1	;for at tiny
	sbi portc,bs1
	nop
	rcall set_din
	nop
	in temp,data_in
	sts tx_data,temp
	rcall serout
	sbi portd,oe
	rjmp main


test_a:
	ldi temp,0
	out portb,temp
	ldi temp,255
	out portb,temp
	rjmp test_a	

set_dout:
	ldi temp,$ff
	out ddrb,temp
	ret

set_din:
	ldi temp,0
	out ddrb,temp
	ret

vdd_on:
	sbi portd,vdd_en
	rjmp main

vpp_on:
	sbi portd,vpp_en
	rjmp main

vdd_off:
	cbi portc,pagel
	cbi portc,xa1
	cbi portc,xa0
	cbi portc,bs1
	rcall set_din
	cbi portd,oe
	cbi portd,xtal1
	cbi portc,wr
	cbi portc,bs2
	cbi portd,vdd_en

	rjmp main

vpp_off:
	cbi portd,vpp_en
	rjmp main


;===========================================================================
byte_delay:
	nop
	nop
	nop
	dec delay_value
	brne byte_delay	
	ret 
		
delay5uS:					;delays at 4MHz clock
	nop
	nop
	nop
	nop
delay4uS:
	nop
	nop
	nop
	nop
delay3uS:
	nop
	nop
	nop
	nop
delay2uS:
	nop
	nop
	nop
	nop
delay1uS:
	ret
									

delay10:
	rcall delay4uS
;	rcall delay5uS
;	rcall delay3uS
	ret
			
delay10uS:
		sts usec10,delay
	
d10usA:
	rcall delay10
	lds delay,usec10
	dec delay
	sts usec10,delay
	brne d10usA
	ret

delayms:
	sts msec,delay
dmsa:
	ldi delay,100
	rcall delay10uS

	lds delay,msec
	dec delay
	sts msec,delay
	brne dmsa
	ret

delayms10:
	sts ms10,delay
dms10a:
	ldi delay,10
	rcall delayms

	lds delay,ms10
	dec delay
	sts ms10,delay
	brne dms10a
	ret

delaysec:
	sts secs,delay
dsecsa:
	ldi delay,100
	rcall delayms10

	lds delay,secs
	dec delay
	sts secs,delay
	brne dsecsa
	ret


WDT_off:
			; Turn off global interrupt
	cli
			; Reset Watchdog Timer
	wdr
			; Clear WDRF in MCUSR
	in r16, MCUSR
	andi r16, (0xff & (0<<WDRF))
	out MCUSR, r16
			; Write logical one to WDCE and WDE
			; Keep old prescaler setting to prevent unintentional time-out
	lds r16, WDTCSR
	ori r16, (1<<WDCE) | (1<<WDE)
	sts WDTCSR, r16
			; Turn off WDT
	ldi r16, (0<<WDE)
	sts WDTCSR, r16
			; Turn on global interrupt
;	sei
	ret



debug:
	cbi portb,reset
	sbi portb,reset
	rjmp debug



