;	serial interface pc to AVR programmer board
;	modified 14/2/2011
;
	
	#include p16f88.inc
	list p=p16f88
;
;================================================================
;             Main programme variable block
;=================================================================
;
;DELAY VARIABLES

	uSec10 equ H'20'
	ms1 equ h'21'
	sec1 equ h'22'
	ms10 equ h'23'
	delay_value equ h'24'

	txdata equ h'25'		;transmit data byte store
	rxdata equ h'26'		;receive data byte store
	bit_counter equ h'27'
	temp equ h'28'

	inbyte equ h'29'

	low_byte equ h'2a'		;data low byte (8 bits)
	high_byte equ h'2b'		;data high byte (6 bits)
	command_byte equ h'2c'		;instruction byte
	byte1 equ h'2d'
	byte2 equ h'2e'
	byte3 equ h'2f'
	byte4 equ h'30'
	indata equ h'31'
	outdata equ h'32'
	byte_count equ h'33'
	echo equ h'34'			;byte echo from AVR write
	out_byte equ h'35'		;byte to be written to AVR
	flags equ h'36'			;flags byte
	sync_count equ h'37'		;prog enable retry count
	debug equ h'38'
;==================================================================
;constants
	baud_rate equ 51 ;12 = 38400  bps 51 = 9600
	prog_sync equ 0		;synchronise bit

	sck equ 0		;RA0 output sck
	reset equ 2		;RA2 output reset
	cts equ 3		;RA2 output CTS
	miso equ 6		;RA6 input
	mosi equ 4		;RA4 output
	led equ 1		;RA1 output led
;==================================================================
;        config byte set up
;=================================================================
	__config _config1, _lvp_off & _boden_off & _mclr_off & _wdt_off & _intrc_io

;=====================================================================

	org 0			;restart vector
	goto  start

	org 4			;location of peripheral interrupt vector
	goto  interrupt		;interrupt routine at this label

	org 6			;where programme actually starts from
;=====================================================================
;=====================================================================
start:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1

	clrf ansel		;allow full access to ports

	movlw b'11100000'
	movwf trisa		;port a inputs 
	movlw b'11011111'	;B2 and B5 usart pins
	movwf trisb		;port b outputs and input

	movlw b'01110000'	;set int rc osc to 8MHz
	movwf osccon

	bcf status,rp0		;select bank 0


;=====================================================================
;Special programme requirements
;=====================================================================

prog_setup:
	call bank1		;bank 1
	movlw baud_rate		;set usart speed
	movwf spbrg		;
	bsf txsta,brgh		;select high speed
	bcf txsta,sync		;tx set up
	bsf txsta,txen		;enable transmitter
	call bank0		;bank 0
	bsf rcsta,spen		;receiver set up
	bcf rcsta,cren		;clear errors and enable receiver
	bsf rcsta,cren

;=====================================================================
;		main programme loop
;=====================================================================

;	call set_prog_mode

	bsf porta,reset		;pre set reset to run mode
	

main_loop:


	bcf porta,cts		;enable serial port
	call serin		;get mode byte
	bsf porta,cts		;disable serial port

	movf rxdata,w		;check if a programmer directive
	sublw 1
	btfsc status,z		;check for read/write mode
	goto set_prog_mode	;reset AVR

	movf rxdata,w		;check if command mode
	sublw 2
	btfsc status,z
	goto prog_enable

	movf rxdata,w		;check if command mode
	sublw 3
	btfsc status,z
	goto set_reset
	

	movf rxdata,w		;check if write mode i.e 4 bytes out
	sublw 4
	btfsc status,z
	goto write_mode

	movf rxdata,w		;check if read mode i.e 3 bytes out and 1 byte in
	sublw 8
	btfsc status,z
	goto read_mode

	movf rxdata,w
	sublw 9
	btfsc status,z
	goto pulse_sck

	goto main_loop		;not a valid command.

;===============================================
;	perform command - may be same as write mode
;===============================================


prog_enable:

	movlw h'ac'
	call write1byte		;send out byte 1
	movlw h'53'
	call write1byte		;send out byte 2
	movlw 0
	call write1byte		;send out byte 3

	movf echo,w
	movwf debug

	movlw 0			;send last byte anyway
	call write1byte
	movf debug,w		
	call serout

	goto main_loop


;===============================================
;	send 3 bytes to AVR and read 1 back
;===============================================

read_mode:
	movlw 3			;read 3 bytes from PC
	call in_n_bytes
	movlw 3			;send 3 bytes to AVR
	call write_n_bytes
	call read_data_byte	;read byte4 from AVR
	movf byte4,w
	call serout		;send byte to PC
	goto main_loop

;===============================================
;	send 4 bytes to AVR
;===============================================

write_mode:
	movlw 4			;read 4 bytes from PC
	call in_n_bytes
	movlw 4			;send 4 bytes to AVR
	call write_n_bytes
	goto main_loop

;===============================================
;	read a byte back from the AVR
;===============================================

read_data_byte:			
	movlw 8			;set up for 8 bits
	movwf bit_counter
	clrf byte4		;clear byte4
loop_read:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bcf status,c		;pre clear carry bit
	btfsc porta,miso	;check state of data bit
	bsf status,c
	rlf byte4,f		;shift data bit in
	bsf porta,sck
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bcf porta,sck
	decfsz bit_counter,f	;loop until 8 bits in
	goto loop_read

	return

;===============================================
;	read 'n' bytes from PC
;===============================================


in_n_bytes:			;come in with byte count in W reg
	movwf byte_count	;save byte count
	movlw byte1		;set index to byte1
	movwf fsr
in_loop:
	bcf porta,cts		;enable serial port
	call serin		;get data byte from PC
	bsf porta,cts		;disable serial port
	movwf indf		;save byte read
	incf fsr,f		;inc index
	decfsz byte_count,f	;loop until 'n' bytes in
	goto in_loop
	return

;================================================
;	send 'n' bytes to AVR
;================================================

write_n_bytes:			;come in with byte count in W reg
	movwf byte_count	;save byte count
	movlw byte1
	movwf fsr		;set index to byte1
write_loop:
	movlw 8			;set bit count to 8
	movwf bit_counter
loop_8_bits:
	btfss indf,7		;test state of data bit
	goto $+3		;go to clear bit
	bsf porta,mosi		;set data bit
	goto $+2
;+3
	bcf porta,mosi		;clear data bit
;+2
	nop
	call pulse_sck1
	nop
	rlf indf,f		;next bit
	decfsz bit_counter,f	;loop until 8 bits out
	goto loop_8_bits
	incf fsr,f		;next byte
	decfsz byte_count,f	;loop until 'n' bytes sent
	goto write_loop
	return


;===============================================
;	write 1 byte to AVR and read echo
;===============================================

write1byte:

	movwf out_byte		;save byte to be written
	movlw 8			;set bit count of 8
	movwf bit_counter
	clrf echo		;echo = 0
write_loop1:
	btfsc out_byte,7	;check state of data bit MSB
	goto setbit
	bcf porta,mosi		;output a 0
	goto clock_bit
setbit:
	bsf porta,mosi		;output a 1
clock_bit:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bsf porta,sck		;clock bit out
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

	bcf status,c		;pre clear carry
	btfsc porta,miso	;read echo bit
	bsf status,c		;set carry as necessary
	rlf echo,f		;clock in bit to echo
	bcf porta,sck
	rlf out_byte,f		;point to next out byte bit
	decfsz bit_counter,f
	goto write_loop1	;loop until 8 bits done
	return



;===============================================
;	read a RS232 byte from PC
;===============================================

serin:
	btfss pir1,rcif		;check if byte received
	goto serin		;not yet
	movf rcreg,w
	movwf rxdata		;save it for later (can be left in rcreg)
	return

;===============================================
;	send a RS232 byte to PC
;===============================================


serout:
	movwf out_byte
	call bank1		;select bank 1
serloop:
	btfss txsta,trmt	;wait for tx empty
	goto serloop
	call bank0		;back to bank 0
	movf out_byte,w		;put data in tx reg
	movwf txreg		;send byte
	return

;===============================================
;	set AVR to programme mode
;===============================================


set_prog_mode:
	bcf porta,sck
	nop
	nop
	nop
	nop
	bsf porta,reset		;zero clock and reset
	nop
	nop
	nop
	nop
	nop
	nop
	nop

	movlw 300
	call delayms
	bcf porta,reset
	movlw 30
	call delayms
	goto main_loop

set_reset:
	bsf porta,reset
	goto main_loop
;===============================================
;	pulse sck high and low
;===============================================

pulse_sck:
	nop
	nop
	bsf porta,sck
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bcf porta,sck
	nop
	nop
	nop
	nop
	nop
	goto main_loop
	

pulse_sck1:
	nop
	nop
	nop
	bsf porta,sck
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bcf porta,sck
	nop
	nop
	nop
	nop
	nop
	return

;=======================================================================
;		general purpose sub routines etc
;=======================================================================

;	bank select routines

bank0:
	bcf status,rp1
	bcf status,rp0
	return

bank1:
	bcf status,rp1
	bsf status,rp0
	return

bank2:
	bsf status,rp1
	bcf status,rp0
	return
bank3:
	bsf status,rp1
	bsf status,rp0
	return

;---------------------------------------
;	delay = 1 usec per count @ 16mhz clock
;	delay = 4 uSec per count @ 4MHz clock
;	count value in w reg at entry
;---------------------------------------

byte_delay:
	movwf delay_value
bd1:
	nop
	nop
	decfsz delay_value,f
	goto bd1
	return

;=======================================================================

delay10:			;10uSec per loop
	movwf usec10		;come in with uSx10 count in w reg
delay10a:
	movlw 4
	call byte_delay

	decfsz usec10,f
	goto delay10a
	return

delayms:
	movwf ms1		;come in with mSec count in W reg
delaymsa:
	movlw 100		;set up for 100 x 10uSec = 1 mSec
	call delay10
	decfsz ms1,f
	goto delaymsa
	return

delayms10:			;come in with mSx10 count in W reg
	movwf ms10
delayms10a:
	movlw 10
	call delayms
	decfsz ms10,f
	goto delayms10a
	return

delaysec:
	movwf sec1		;come in with seconds count in W reg
delayseca:
	movlw 100		;10 mSec
	call delayms10			
	decfsz sec1,f
	goto delayseca
	return


;==========================================================================
;		interrupt routine
;==========================================================================
INTERRUPT:			;Just in case we have an accidental interrupt
	RETFIE
	end			;this is a mandatory command
;==========================================================================
;	end of programme
;==========================================================================
